@@ -38,7 +38,6 @@ bundle = Table(
     UniqueConstraint('uuid', name='uix_1'),
     Index('bundle_data_hash_index', 'data_hash'),
     Index('state_index', 'state'),  # Needed for the bundle manager.
-    sqlite_autoincrement=True,
 )
 
 # Includes things like name, description, etc.
@@ -50,7 +49,6 @@ bundle_metadata = Table(
     Column('metadata_key', String(63), nullable=False),
     Column('metadata_value', Text, nullable=False),
     Index('metadata_kv_index', 'metadata_key', 'metadata_value', mysql_length=63),
-    sqlite_autoincrement=True,
 )
 
 # For each child_uuid, we have: key = child_path, target = (parent_uuid, parent_path)
@@ -64,7 +62,6 @@ bundle_dependency = Table(
     # dependencies to bundles not (yet) in the system.
     Column('parent_uuid', String(63), nullable=False),
     Column('parent_path', Text, nullable=False),
-    sqlite_autoincrement=True,
 )
 
 # The worksheet table does not have many columns now, but it will eventually
@@ -86,7 +83,6 @@ worksheet = Table(
     UniqueConstraint('uuid', name='uix_1'),
     Index('worksheet_name_index', 'name'),
     Index('worksheet_owner_index', 'owner_id'),
-    sqlite_autoincrement=True,
 )
 
 worksheet_item = Table(
@@ -109,7 +105,6 @@ worksheet_item = Table(
     Index('worksheet_item_worksheet_uuid_index', 'worksheet_uuid'),
     Index('worksheet_item_bundle_uuid_index', 'bundle_uuid'),
     Index('worksheet_item_subworksheet_uuid_index', 'subworksheet_uuid'),
-    sqlite_autoincrement=True,
 )
 
 # Worksheet tags
@@ -121,7 +116,6 @@ worksheet_tag = Table(
     Column('tag', String(63), nullable=False),
     Index('worksheet_tag_worksheet_uuid_index', 'worksheet_uuid'),
     Index('worksheet_tag_tag_index', 'tag'),
-    sqlite_autoincrement=True,
 )
 
 group = Table(
@@ -135,7 +129,6 @@ group = Table(
     UniqueConstraint('uuid', name='uix_1'),
     Index('group_name_index', 'name'),
     Index('group_owner_id_index', 'owner_id'),
-    sqlite_autoincrement=True,
 )
 
 user_group = Table(
@@ -148,7 +141,6 @@ user_group = Table(
     Column('is_admin', Boolean),
     Index('group_uuid_index', 'group_uuid'),
     Index('user_id_index', 'user_id'),
-    sqlite_autoincrement=True,
 )
 
 # Permissions for bundles
@@ -161,7 +153,6 @@ group_bundle_permission = Table(
     Column('object_uuid', String(63), ForeignKey(bundle.c.uuid), nullable=False),
     # Permissions encoded as integer (see below)
     Column('permission', Integer, nullable=False),
-    sqlite_autoincrement=True,
 )
 
 # Permissions for worksheets
@@ -174,7 +165,6 @@ group_object_permission = Table(
     Column('object_uuid', String(63), ForeignKey(worksheet.c.uuid), nullable=False),
     # Permissions encoded as integer (see below)
     Column('permission', Integer, nullable=False),
-    sqlite_autoincrement=True,
 )
 
 # A permission value is one of the following: none (0), read (1), or all (2).
@@ -182,33 +172,6 @@ GROUP_OBJECT_PERMISSION_NONE = 0x00
 GROUP_OBJECT_PERMISSION_READ = 0x01
 GROUP_OBJECT_PERMISSION_ALL = 0x02
 
-# Keep track of events that happen.
-event = Table(
-    'event',
-    db_metadata,
-    Column('id', Integer, primary_key=True, nullable=False),
-    Column(
-        'date', String(63), nullable=False
-    ),  # Deterministic function (e.g., 2015-09-11) of the start_time
-    Column('start_time', DateTime, nullable=False),  # When did this event start?
-    Column('end_time', DateTime, nullable=False),  # When did this event end?
-    Column('duration', Float, nullable=False),  # How much time did this event take?
-    Column('user_id', String(63), nullable=True),  # Who did it?
-    Column('user_name', String(63), nullable=True),  # Who did it?
-    Column('command', String(63), nullable=False),  # The command (gotten in bundle_rpc_server)
-    Column('args', Text, nullable=False),  # JSON string
-    Column(
-        'uuid', String(63), nullable=True
-    ),  # Either bundle or worksheet id (no ForeignKey because might not be in system anymore)
-    # Indices
-    Index('events_date_index', 'date'),
-    Index('events_user_id_index', 'user_id'),
-    Index('events_user_name_index', 'user_name'),
-    Index('events_command_index', 'command'),
-    Index('events_uuid_index', 'uuid'),
-    sqlite_autoincrement=True,
-)
-
 # A notifications value is one of the following:
 NOTIFICATIONS_NONE = 0x00  # Receive no notifications
 NOTIFICATIONS_IMPORTANT = 0x01  # Receive only important notifications
@@ -243,13 +206,13 @@ user = Table(
     Column('url', String(255, convert_unicode=True), nullable=True),
     # Quotas
     Column('time_quota', Float, nullable=False),  # Number of seconds allowed
+    Column('parallel_run_quota', Integer, nullable=False),  # Number of parallel jobs allowed
     Column('time_used', Float, nullable=False),  # Number of seconds already used
     Column('disk_quota', Float, nullable=False),  # Number of bytes allowed
     Column('disk_used', Float, nullable=False),  # Number of bytes already used
     Index('user_user_id_index', 'user_id'),
     Index('user_user_name_index', 'user_name'),
     UniqueConstraint('user_id', name='uix_1'),
-    sqlite_autoincrement=True,
 )
 
 # Stores (email) verification keys
@@ -261,7 +224,6 @@ user_verification = Table(
     Column('date_created', DateTime, nullable=False),
     Column('date_sent', DateTime, nullable=True),
     Column('key', String(64), nullable=False),
-    sqlite_autoincrement=True,
 )
 
 # Stores password reset codes
@@ -272,7 +234,6 @@ user_reset_code = Table(
     Column('user_id', String(63), ForeignKey(user.c.user_id), nullable=False),
     Column('date_created', DateTime, nullable=False),
     Column('code', String(64), nullable=False),
-    sqlite_autoincrement=True,
 )
 
 # OAuth2 Tables
@@ -294,7 +255,6 @@ oauth2_client = Table(
     Column('scopes', Text, nullable=False),  # comma-separated list of allowed scopes
     Column('redirect_uris', Text, nullable=False),  # comma-separated list of allowed redirect URIs
     UniqueConstraint('client_id', name='uix_1'),
-    sqlite_autoincrement=True,
 )
 
 oauth2_token = Table(
@@ -307,7 +267,6 @@ oauth2_token = Table(
     Column('access_token', String(255), unique=True),
     Column('refresh_token', String(255), unique=True),
     Column('expires', DateTime, nullable=False),
-    sqlite_autoincrement=True,
 )
 
 oauth2_auth_code = Table(
@@ -320,7 +279,6 @@ oauth2_auth_code = Table(
     Column('code', String(100), nullable=False),
     Column('expires', DateTime, nullable=False),
     Column('redirect_uri', String(255), nullable=False),
-    sqlite_autoincrement=True,
 )
 
 # Store information about users' questions or feedback.
@@ -338,7 +296,6 @@ chat = Table(
     Column(
         'bundle_uuid', String(63), nullable=True
     ),  # What is the id of the bundle that the sender is on?
-    sqlite_autoincrement=True,
 )
 
 # Store information about workers.
@@ -351,10 +308,17 @@ worker = Table(
     Column('cpus', Integer, nullable=False),  # Number of CPUs on worker.
     Column('gpus', Integer, nullable=False),  # Number of GPUs on worker.
     Column('memory_bytes', BigInteger, nullable=False),  # Total memory of worker.
+    Column('free_disk_bytes', BigInteger, nullable=True),  # Available disk space on worker.
     Column(
         'checkin_time', DateTime, nullable=False
     ),  # When the worker last checked in with the bundle service.
     Column('socket_id', Integer, nullable=False),  # Socket ID worker listens for messages on.
+    Column(
+        'shared_file_system', Boolean, nullable=False
+    ),  # Whether the worker and the server have a shared filesystem.
+    Column(
+        'tag_exclusive', Boolean, nullable=False
+    ),  # Whether worker runs bundles if and only if they match tags.
 )
 
 # Store information about all sockets currently allocated to each worker.
@@ -389,5 +353,4 @@ worker_dependency = Table(
     # Serialized list of dependencies for the user/worker combination.
     # See WorkerModel for the serialization method.
     Column('dependencies', LargeBinary, nullable=False),
-    sqlite_autoincrement=True,
 )
