@@ -14,6 +14,7 @@ import types
 from contextlib import closing
 from itertools import chain
 import json
+import sys
 
 import yaml
 from bottle import get, post, local, request
@@ -39,6 +40,7 @@ from codalab.objects.permission import permission_str
 from codalab.rest import util as rest_util
 from codalab.rest.worksheets import get_worksheet_info, search_worksheets
 from codalab.rest.worksheet_block_schemas import BlockModes, MarkupBlockSchema, FetchStatusCodes
+from codalab.worker.download_util import BundleTarget
 
 
 @post('/interpret/search')
@@ -156,25 +158,40 @@ def fetch_interpreted_worksheet(uuid):
     - resolve_interpreted_items: get more information about a worksheet.
     In the future, for large worksheets, might want to break this up so
     that we can render something basic.
+
+    This endpoint can be called with &brief=1 in order to give an abbreviated version,
+    which does not resolve searches or wsearches.
+
+    To return an interpreted worksheet that only resolves a particular search/wsearch,
+    pass in the search query to the "directive" argument. The value for this argument
+    must be a search/wsearch query -- for example, &directive=search 0x .limit=100
     """
     bundle_uuids = request.query.getall('bundle_uuid')
+    brief = request.query.get("brief", "0") == "1"
+
+    directive = request.query.get("directive", None)
+
     worksheet_info = get_worksheet_info(uuid, fetch_items=True, fetch_permissions=True)
 
-    # Shim in additional data for the frontend
-    worksheet_info['items'] = resolve_items_into_infos(worksheet_info['items'])
+    if directive:
+        # Only perform a single search query.
+        worksheet_info['items'] = perform_search_query(formatting.string_to_tokens(directive))
+    else:
+        # Shim in additional data for the frontend
+        worksheet_info['items'] = resolve_items_into_infos(worksheet_info['items'])
+        worksheet_info['raw'] = get_worksheet_lines(worksheet_info)
+
     if worksheet_info['owner_id'] is None:
         worksheet_info['owner_name'] = None
     else:
         owner = local.model.get_user(user_id=worksheet_info['owner_id'])
         worksheet_info['owner_name'] = owner.user_name
 
-    # Fetch items.
-    worksheet_info['raw'] = get_worksheet_lines(worksheet_info)
-
     # Replace searches with raw items.
     # This needs to be done before get_worksheet_lines because this replaces
     # user-written raw items.
-    worksheet_info['items'] = expand_raw_items(worksheet_info['items'])
+    if not directive and not brief:
+        worksheet_info['items'] = expand_search_items(worksheet_info['items'])
 
     # Set permissions
     worksheet_info['edit_permission'] = worksheet_info['permission'] == GROUP_OBJECT_PERMISSION_ALL
@@ -188,7 +205,9 @@ def fetch_interpreted_worksheet(uuid):
     # Go and fetch more information about the worksheet contents by
     # resolving the interpreted items.
     try:
-        interpreted_blocks = interpret_items(get_default_schemas(), worksheet_info['items'])
+        interpreted_blocks = interpret_items(
+            get_default_schemas(), worksheet_info['items'], db_model=local.model
+        )
     except UsageError as e:
         interpreted_blocks = {'blocks': []}
         worksheet_info['error'] = str(e)
@@ -223,7 +242,7 @@ def fetch_interpreted_worksheet(uuid):
             continue
         if item['mode'] == 'table':
             for row_map in item['rows']:
-                for k, v in row_map.iteritems():
+                for k, v in row_map.items():
                     if v is None:
                         row_map[k] = formatting.contents_str(v)
         if 'bundle_info' in item:
@@ -255,9 +274,7 @@ def cat_target(target):
     The caller should ensure that the target is a file.
     """
     rest_util.check_target_has_read_permission(target)
-    with closing(
-        local.download_manager.stream_file(target[0], target[1], gzipped=False)
-    ) as fileobj:
+    with closing(local.download_manager.stream_file(target, gzipped=False)) as fileobj:
         return fileobj.read()
 
 
@@ -265,23 +282,24 @@ def cat_target(target):
 MAX_BYTES_PER_LINE = 1024
 
 
-def head_target(target, max_num_lines, replace_non_unicode=False):
+def head_target(target, max_num_lines):
     """
     Return the first max_num_lines of target as a list of strings.
 
     The caller should ensure that the target is a file.
 
-    :param target: (bundle_uuid, subpath)
+    :param target: A worker.download_util.BundleTarget with bundle_uuid and subpath
     :param max_num_lines: max number of lines to fetch
-    :param replace_non_unicode: replace non-unicode characters with something printable
     """
     rest_util.check_target_has_read_permission(target)
-    lines = local.download_manager.summarize_file(
-        target[0], target[1], max_num_lines, 0, MAX_BYTES_PER_LINE, None, gzipped=False
-    ).splitlines(True)
-
-    if replace_non_unicode:
-        lines = map(formatting.verbose_contents_str, lines)
+    # Note: summarize_file returns bytes, but should be decodable to a string.
+    lines = (
+        local.download_manager.summarize_file(
+            target, max_num_lines, 0, MAX_BYTES_PER_LINE, None, gzipped=False
+        )
+        .decode()
+        .splitlines(True)
+    )
 
     return lines
 
@@ -310,7 +328,7 @@ def resolve_interpreted_blocks(interpreted_blocks):
 
         try:
             # Replace data with a resolved version.
-            if mode == BlockModes.markup_block:
+            if mode in (BlockModes.markup_block, BlockModes.placeholder_block):
                 # no need to do anything
                 pass
             elif mode == BlockModes.record_block or mode == BlockModes.table_block:
@@ -321,11 +339,11 @@ def resolve_interpreted_blocks(interpreted_blocks):
 
                 block['rows'] = contents
             elif mode == BlockModes.contents_block or mode == BlockModes.image_block:
+                bundle_uuid = block['bundles_spec']['bundle_infos'][0]['uuid']
+                target_path = block['target_genpath']
+                target = BundleTarget(bundle_uuid, target_path)
                 try:
-                    target_info = rest_util.get_target_info(
-                        (block['bundles_spec']['bundle_infos'][0]['uuid'], block['target_genpath']),
-                        0,
-                    )
+                    target_info = rest_util.get_target_info(target, 0)
                     if target_info['type'] == 'directory' and mode == BlockModes.contents_block:
                         block['status']['code'] = FetchStatusCodes.ready
                         block['lines'] = ['<directory>']
@@ -333,23 +351,13 @@ def resolve_interpreted_blocks(interpreted_blocks):
                         block['status']['code'] = FetchStatusCodes.ready
                         if mode == BlockModes.contents_block:
                             block['lines'] = head_target(
-                                (
-                                    block['bundles_spec']['bundle_infos'][0]['uuid'],
-                                    block['target_genpath'],
-                                ),
-                                block['max_lines'],
-                                replace_non_unicode=True,
+                                target_info['resolved_target'], block['max_lines']
                             )
                         elif mode == BlockModes.image_block:
                             block['status']['code'] = FetchStatusCodes.ready
                             block['image_data'] = base64.b64encode(
-                                cat_target(
-                                    (
-                                        block['bundles_spec']['bundle_infos'][0]['uuid'],
-                                        block['target_genpath'],
-                                    )
-                                )
-                            )
+                                bytes(cat_target(target_info['resolved_target']))
+                            ).decode('utf-8')
                     else:
                         block['status']['code'] = FetchStatusCodes.not_found
                         if mode == BlockModes.contents_block:
@@ -367,13 +375,13 @@ def resolve_interpreted_blocks(interpreted_blocks):
                 # data = list of {'target': ...}
                 # Add a 'points' field that contains the contents of the target.
                 for info in block['trajectories']:
-                    target = (info['bundle_uuid'], info['target_genpath'])
+                    target = BundleTarget(info['bundle_uuid'], info['target_genpath'])
                     try:
                         target_info = rest_util.get_target_info(target, 0)
                     except NotFoundError as e:
                         continue
                     if target_info['type'] == 'file':
-                        contents = head_target(target, block['max_lines'], replace_non_unicode=True)
+                        contents = head_target(target_info['resolved_target'], block['max_lines'])
                         # Assume TSV file without header for now, just return each line as a row
                         info['points'] = points = []
                         for line in contents:
@@ -386,9 +394,9 @@ def resolve_interpreted_blocks(interpreted_blocks):
                 raise UsageError('Invalid display mode: %s' % mode)
 
         except UsageError as e:
-            set_error_data(block_index, e.message)
+            set_error_data(block_index, str(e))
 
-        except StandardError:
+        except Exception:
             import traceback
 
             traceback.print_exc()
@@ -401,7 +409,7 @@ def resolve_interpreted_blocks(interpreted_blocks):
 
 def is_bundle_genpath_triple(value):
     # if called after an RPC call tuples may become lists
-    need_gen_types = (types.TupleType, types.ListType)
+    need_gen_types = (tuple, list)
 
     return isinstance(value, need_gen_types) and len(value) == 3
 
@@ -470,13 +478,13 @@ def interpret_file_genpath(target_cache, bundle_uuid, genpath, post):
     else:
         subpath, key = genpath, None
 
-    target = (bundle_uuid, subpath)
+    target = BundleTarget(bundle_uuid, subpath)
     if target not in target_cache:
         info = None
         try:
             target_info = rest_util.get_target_info(target, 0)
             if target_info['type'] == 'file':
-                contents = head_target(target, MAX_LINES)
+                contents = head_target(target_info['resolved_target'], MAX_LINES)
 
                 if len(contents) == 0:
                     info = ''
@@ -536,7 +544,7 @@ def resolve_items_into_infos(items):
     # optimized batch_get_bundles multiget method.
     bundle_uuids = set(i['bundle_uuid'] for i in items if i['bundle_uuid'] is not None)
 
-    bundle_dict = rest_util.get_bundle_infos(bundle_uuids)
+    bundle_dict = rest_util.get_bundle_infos(bundle_uuids, get_single_host_worksheet=True)
 
     # Go through the items and substitute the components
     new_items = []
@@ -546,6 +554,9 @@ def resolve_items_into_infos(items):
             if i['bundle_uuid']
             else None
         )
+        if bundle_info is not None:
+            bundle_info['sort_key'] = i['sort_key']
+            bundle_info['id'] = i['id']
         if i['subworksheet_uuid']:
             try:
                 subworksheet_info = local.model.get_worksheet(
@@ -561,15 +572,56 @@ def resolve_items_into_infos(items):
         value_obj = (
             formatting.string_to_tokens(i['value']) if i['type'] == TYPE_DIRECTIVE else i['value']
         )
-        new_items.append((bundle_info, subworksheet_info, value_obj, i['type']))
+        new_items.append(
+            (bundle_info, subworksheet_info, value_obj, i['type'], i['id'], i['sort_key'])
+        )
     return new_items
 
 
-def expand_raw_items(raw_items):
-    return list(chain.from_iterable([expand_raw_item(raw_item) for raw_item in raw_items]))
+def perform_search_query(value_obj):
+    """
+    Perform a search query and return the resulting raw items.
+    Input: directive that is tokenized by formatting.string_to_tokens(),
+        such as formatting.string_to_tokens("search 0x .limit=100")
+    """
+    command = get_command(value_obj)
+    is_search = command == 'search'
+    is_wsearch = command == 'wsearch'
+    if is_search or is_wsearch:
+        keywords = value_obj[1:]
+        raw_items = []
+
+        if is_search:
+            keywords = rest_util.resolve_owner_in_keywords(keywords)
+            search_result = local.model.search_bundles(request.user.user_id, keywords)
+            if search_result['is_aggregate']:
+                # Add None's for the 'id' and 'sort_key' of the tuple, since these
+                # items are not really worksheet items.
+                raw_items.append(markup_item(str(search_result['result'])) + (None, None))
+            else:
+                bundle_uuids = search_result['result']
+                bundle_infos = rest_util.get_bundle_infos(bundle_uuids)
+                for bundle_uuid in bundle_uuids:
+                    # Since bundle UUID's are queried first, we can't assume a UUID exists in
+                    # the subsequent bundle info query.
+                    if bundle_uuid in bundle_infos:
+                        raw_items.append(bundle_item(bundle_infos[bundle_uuid]) + (None, None))
+        elif is_wsearch:
+            worksheet_infos = search_worksheets(keywords)
+            for worksheet_info in worksheet_infos:
+                raw_items.append(subworksheet_item(worksheet_info) + (None, None))
+
+        return raw_items
+    else:
+        # Not a search query
+        return []
 
 
-def expand_raw_item(raw_item):
+def expand_search_items(raw_items):
+    return list(chain.from_iterable([expand_search_item(raw_item) for raw_item in raw_items]))
+
+
+def expand_search_item(raw_item):
     """
     Raw items that include searches must be expanded into more raw items.
     Input: Raw item.
@@ -578,32 +630,8 @@ def expand_raw_item(raw_item):
     otherwise it contains the search result. You do not need to call
     resolve_items_into_infos on the returned raw_items.
     """
-
-    (bundle_info, subworksheet_info, value_obj, item_type) = raw_item
-
-    is_search = item_type == TYPE_DIRECTIVE and get_command(value_obj) == 'search'
-    is_wsearch = item_type == TYPE_DIRECTIVE and get_command(value_obj) == 'wsearch'
-
-    if is_search or is_wsearch:
-        command = get_command(value_obj)
-        keywords = value_obj[1:]
-        raw_items = []
-
-        if is_search:
-            keywords = rest_util.resolve_owner_in_keywords(keywords)
-            search_result = local.model.search_bundles(request.user.user_id, keywords)
-            if search_result['is_aggregate']:
-                raw_items.append(markup_item(str(search_result['result'])))
-            else:
-                bundle_uuids = search_result['result']
-                bundle_infos = rest_util.get_bundle_infos(bundle_uuids)
-                for bundle_uuid in bundle_uuids:
-                    raw_items.append(bundle_item(bundle_infos[bundle_uuid]))
-        elif is_wsearch:
-            worksheet_infos = search_worksheets(keywords)
-            for worksheet_info in worksheet_infos:
-                raw_items.append(subworksheet_item(worksheet_info))
-
-        return raw_items
+    (bundle_info, subworksheet_info, value_obj, item_type, id, sort_key) = raw_item
+    if item_type == TYPE_DIRECTIVE and get_command(value_obj) in ('search', 'wsearch'):
+        return perform_search_query(value_obj)
     else:
         return [raw_item]
